#include <ros/ros.h>
#include <dynamic_reconfigure/server.h>
#include <hark_dynamic_reconfigure/StdMsgsConfig.h>

#include <std_msgs/Bool.h>
#include <std_msgs/Float32.h>
#include <std_msgs/Float64.h>
#include <std_msgs/Int8.h>
#include <std_msgs/Int16.h>
#include <std_msgs/Int32.h>
#include <std_msgs/Int64.h>
#include <std_msgs/String.h>
#include <std_msgs/UInt8.h>
#include <std_msgs/UInt16.h>
#include <std_msgs/UInt32.h>
#include <std_msgs/UInt64.h>

ros::Publisher* g_pBoolPublisher;

ros::Publisher* g_pStringPublisher;

ros::Publisher* g_pFloat32Publisher;
ros::Publisher* g_pFloat64Publisher;

ros::Publisher* g_pInt8Publisher;
ros::Publisher* g_pInt16Publisher;
ros::Publisher* g_pInt32Publisher;
ros::Publisher* g_pInt64Publisher;

ros::Publisher* g_pUInt8Publisher;
ros::Publisher* g_pUInt16Publisher;
ros::Publisher* g_pUInt32Publisher;
ros::Publisher* g_pUInt64Publisher;

void callback(hark_dynamic_reconfigure::StdMsgsConfig &config, uint32_t level)
{
  
  std_msgs::Bool oBool;
  std_msgs::String oString;

  std_msgs::Float32 oFloat32;
  std_msgs::Float64 oFloat64;

  std_msgs::Int8 oInt8;
  std_msgs::Int16 oInt16;
  std_msgs::Int32 oInt32;
  std_msgs::Int64 oInt64;

  std_msgs::UInt8  oUInt8;
  std_msgs::UInt16 oUInt16;
  std_msgs::UInt32 oUInt32;
  std_msgs::UInt64 oUInt64;

  oBool.data = config.bool_;
  oString.data = config.str_;
  
  oFloat32.data = (float)config.double_;
  oFloat64.data = (double)config.double_;

  oInt8.data  = (signed char)config.int_;
  oInt16.data = (signed short)config.int_;
  oInt32.data = (signed int)config.int_;
  oInt64.data = (signed long)config.int_;

  oUInt8.data  = (unsigned char)config.int_;
  oUInt16.data = (unsigned short)config.int_;
  oUInt32.data = (unsigned int)config.int_;
  oUInt64.data = (unsigned long)config.int_;
  
  if(config.publish_flag){

    g_pBoolPublisher->publish( oBool );

    g_pStringPublisher->publish( oString );

    g_pFloat32Publisher->publish( oFloat32 );
    g_pFloat64Publisher->publish( oFloat64 );

    g_pInt8Publisher->publish( oInt8 );
    g_pInt16Publisher->publish( oInt16 );
    g_pInt32Publisher->publish( oInt32 );
    g_pInt64Publisher->publish( oInt64 );

    g_pUInt8Publisher->publish(  oUInt8 );
    g_pUInt16Publisher->publish( oUInt16 );
    g_pUInt32Publisher->publish( oUInt32 );
    g_pUInt64Publisher->publish( oUInt64 );

    config.publish_flag = !config.publish_flag;
   
  }

}

int main(int argc, char **argv)
{
  ros::init(argc, argv, "std_msgs_publisher_by_dynamic_reconfigure");
  ros::NodeHandle n;

  dynamic_reconfigure::Server<hark_dynamic_reconfigure::StdMsgsConfig> srv;
  dynamic_reconfigure::Server<hark_dynamic_reconfigure::StdMsgsConfig>::CallbackType f = boost::bind(&callback, _1, _2);

  ros::Publisher oBoolPublisher = n.advertise<std_msgs::Bool>( "/hark/Bool", 100 );

  ros::Publisher oStringPublisher = n.advertise<std_msgs::String>( "/hark/String", 100 );

  ros::Publisher oFloat32Publisher = n.advertise<std_msgs::Float32>( "/hark/Float32", 100 );
  ros::Publisher oFloat64Publisher = n.advertise<std_msgs::Float64>( "/hark/Float64", 100 );

  ros::Publisher oInt8Publisher  = n.advertise<std_msgs::Int8>(  "/hark/Int8", 100 );
  ros::Publisher oInt16Publisher = n.advertise<std_msgs::Int16>( "/hark/Int16", 100 );
  ros::Publisher oInt32Publisher = n.advertise<std_msgs::Int32>( "/hark/Int32", 100 );
  ros::Publisher oInt64Publisher = n.advertise<std_msgs::Int64>( "/hark/Int64", 100 );

  ros::Publisher oUInt8Publisher  = n.advertise<std_msgs::UInt8>(  "/hark/UInt8", 100 );
  ros::Publisher oUInt16Publisher = n.advertise<std_msgs::UInt16>( "/hark/UInt16", 100 );
  ros::Publisher oUInt32Publisher = n.advertise<std_msgs::UInt32>( "/hark/UInt32", 100 );
  ros::Publisher oUInt64Publisher = n.advertise<std_msgs::UInt64>( "/hark/UInt64", 100 );

  g_pBoolPublisher = &oBoolPublisher;

  g_pStringPublisher = &oStringPublisher;

  g_pFloat32Publisher = &oFloat32Publisher;
  g_pFloat64Publisher = &oFloat64Publisher;

  g_pInt8Publisher  = &oInt8Publisher;
  g_pInt16Publisher = &oInt16Publisher;
  g_pInt32Publisher = &oInt32Publisher;
  g_pInt64Publisher = &oInt64Publisher;

  g_pUInt8Publisher  = &oUInt8Publisher;
  g_pUInt16Publisher = &oUInt16Publisher;
  g_pUInt32Publisher = &oUInt32Publisher;
  g_pUInt64Publisher = &oUInt64Publisher;

  srv.setCallback(f);
  ROS_INFO("Starting to spin...");
  ros::spin();
  return 0;
}
